package by.bs.gefdemo;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.util.Locale;

import javax.swing.UIManager;

import org.tigris.gef.util.Localizer;
import org.tigris.gef.util.ResourceLoader;

/**
 *  Main entry point module for gef demo application
 *
 *@author     S.Berdachuk
 *@created    Dec 2003.
 */
public class Main {
    /**
     *  Constructor for the Main object
     *
     *@param  args  - application params
     */
    public Main(String[] args) {
        if (!CheckParams(args)) {
            System.exit(0);
        } else {
            try {
                UIManager.setLookAndFeel(selectedLookAndFeel);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            //mapping resources
            Localizer.addResource("GefBase",
                    "org.tigris.gef.base.BaseResourceBundle");
            Localizer.addResource("GefPres",
                    "org.tigris.gef.presentation.PresentationResourceBundle");
            ResourceLoader.addResourceExtension("gif");
            ResourceLoader.addResourceLocation("/org/tigris/gef/Images");
            ResourceLoader.addResourceLocation("/by/bs/gefdemo/resource");
            Localizer.addResource("GefDemo",
                    "by.bs.gefdemo.resource.GefDemo");
            //set default user locale
            Locale locale = Locale.getDefault();
            Localizer.addLocale(locale);
            Localizer.switchCurrentLocale(locale);

            frame = new MainFrame();
            //allign frame
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            Dimension frameSize = frame.getSize();
            frameSize.width = (int) (0.95 * screenSize.width);
            frameSize.height = (int) (0.90 * screenSize.height);
            frame.setLocation(
                    (screenSize.width - frameSize.width) / 2,
                    (screenSize.height - frameSize.height) / 2);
            frame.setSize(frameSize);

            frame.addWindowListener(
                new WindowAdapter() {

                    public void windowClosing(WindowEvent event) {
                        frame.dispose();
                    }


                    public void windowClosed(WindowEvent event) {
                        System.exit(0);
                    }
                });
            frame.setVisible(true);
        }
    }


    /**
     *  main entry point
     *
     *@param  args  - arguments
     */
    public static void main(String args[]) {
        new Main(args);
    }


    /**
     *  Check application parameters
     *
     *@param  args  - application arguments
     *@return       true if params check ok
     */
    protected static boolean CheckParams(String args[]) {
        try {
            for (int i = 0; i < args.length; i++) {
                String s = args[i];
                char c1 = s.charAt(0);
                if (c1 != '-' && c1 != '/') {
                    return false;
                }
                if (s.length() > 1) {
                    String s1 = s.substring(1);
                    if (s1.startsWith("ui=")) {
                        if (s1.length() > 2) {
                            if (s1.substring(3).equals("windows") && (System.getProperty("os.name").startsWith("Windows"))) {
                                selectedLookAndFeel = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
                            } else if (s1.substring(3).equals("motif")) {
                                selectedLookAndFeel = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
                            } else if (s1.substring(3).equals("metal")) {
                                selectedLookAndFeel = "javax.swing.plaf.metal.MetalLookAndFeel";
                            } else {
                                selectedLookAndFeel = UIManager.getSystemLookAndFeelClassName();
                            }
                        } else {
                            return false;
                        }
                    }
                } else {
                    return false;
                }
            }
        } catch (Exception exception) {
            return false;
        }
        return true;
    }


    private MainFrame frame;
    private static String selectedLookAndFeel = UIManager.getSystemLookAndFeelClassName();
}