package by.bs.gefdemo;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.border.EtchedBorder;

import org.tigris.gef.base.CmdAdjustGrid;
import org.tigris.gef.base.CmdAdjustGuide;
import org.tigris.gef.base.CmdAdjustPageBreaks;
import org.tigris.gef.base.CmdAlign;
import org.tigris.gef.base.CmdCopy;
import org.tigris.gef.base.CmdDelete;
import org.tigris.gef.base.CmdExit;
import org.tigris.gef.base.CmdGroup;
import org.tigris.gef.base.CmdOpenPGML;
import org.tigris.gef.base.CmdPaste;
import org.tigris.gef.base.CmdPrint;
import org.tigris.gef.base.CmdPrintPageSetup;
import org.tigris.gef.base.CmdReorder;
import org.tigris.gef.base.CmdSavePGML;
import org.tigris.gef.base.CmdSelectAll;
import org.tigris.gef.base.CmdSelectInvert;
import org.tigris.gef.base.CmdSelectNext;
import org.tigris.gef.base.CmdUngroup;
import org.tigris.gef.base.Editor;
import org.tigris.gef.base.Globals;
import org.tigris.gef.demo.SamplePalette;
import org.tigris.gef.graph.GraphEdgeRenderer;
import org.tigris.gef.graph.GraphModel;
import org.tigris.gef.graph.GraphNodeRenderer;
import org.tigris.gef.graph.presentation.JGraph;
import org.tigris.gef.ui.IStatusBar;
import org.tigris.gef.ui.PaletteFig;
import org.tigris.gef.util.Localizer;

/**
 *  Main application frame
 *
 *@author     S. Berdachuk
 *@created    Dec 2003.
 */
public class MainFrame extends JFrame
         implements IStatusBar, Cloneable {

    /**
     *  Constructor for the MainFrame object
     */
    public MainFrame() {
        this("Gef demo");
    }


    /**
     *  Constructor for the MainFrame object
     *
     *@param  title  - frame title
     */
    public MainFrame(String title) {
        this(title, new JGraph());
    }


    /**
     *  Constructor for the MainFrame object
     *
     *@param  title  - frame title
     *@param  ed     - graph editor
     */
    public MainFrame(String title, Editor ed) {
        this(title, new JGraph(ed));
    }


    /**
     *  Constructor for the MainFrame object
     *
     *@param  title  - frame title
     *@param  graph  
     */
    public MainFrame(String title, JGraph graph) {
        super(title);
        fGraph = graph;
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     *  Constructor for the MainFrame object
     *
     *@param  gm  graph model
     */
    public MainFrame(GraphModel gm) {
        this("untitled");
        setGraphModel(gm);
    }


    /**
     *  Objects initialisation.
     *  For supporting GUI editors created as external method
     *
     *@exception  Exception  
     */
    private void jbInit() throws Exception {

        //setUp Menus
        this.setSize(new Dimension(401, 257));

        mmFile.setText(Localizer.localize("GefDemo", "MainFrame.mmFile"));
        mmFile.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmFileMnemonic").charAt(0));
        fMenuBar.add(mmFile);

        mmiOpenPGML.setText(Localizer.localize("GefDemo", "MainFrame.mmiFileOpenPGML"));
        mmiOpenPGML.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmiFileOpenPGMLMnemonic").charAt(0));
        //This action isn't work properly, need revision
        //mmFile.add(mmiOpenPGML);

        KeyStroke ctrlS = KeyStroke.getKeyStroke(83, 2);
        mmiSavePGML.setText(Localizer.localize("GefDemo", "MainFrame.mmiFileSavePGML"));
        mmiSavePGML.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmiFileSavePGMLMnemonic").charAt(0));
        mmiSavePGML.setAccelerator(ctrlS);
        //mmFile.add(mmiSavePGML);
        //mmFile.addSeparator();

        KeyStroke ctrlP = KeyStroke.getKeyStroke(80, 2);
        mmiPrint.setAccelerator(ctrlP);
        mmiPrint.setText(Localizer.localize("GefDemo", "MainFrame.mmiPrint"));
        mmiPrint.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmiPrintMnemonic").charAt(0));
        mmFile.add(mmiPrint);

        mmiPageSetup.setText(Localizer.localize("GefDemo", "MainFrame.mmiPageSetup"));
        mmiPageSetup.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmiPageSetupMnemonic").charAt(0));
        mmFile.add(mmiPageSetup);
        mmFile.addSeparator();

        KeyStroke altF4 = KeyStroke.getKeyStroke(115, 8);
        mmiExit.setAccelerator(altF4);
        mmiExit.setText(Localizer.localize("GefDemo", "MainFrame.mmiExit"));
        mmiExit.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmiExitMnemonic").charAt(0));
        mmFile.add(mmiExit);

        mmEdit.setText(Localizer.localize("GefDemo", "MainFrame.mmiEdit"));
        mmEdit.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmiEditMnemonic").charAt(0));
        fMenuBar.add(mmEdit);

        KeyStroke ctrlA = KeyStroke.getKeyStroke(65, 2);
        mmiSelectAll.setAccelerator(ctrlA);
        mmiSelectAll.setText(Localizer.localize("GefDemo", "MainFrame.mmSelectAll"));
        mmEdit.add(mmiSelectAll);

        mmiSelectPrev.setText(Localizer.localize("GefDemo", "MainFrame.mmiSelectPrev"));
        mmEdit.add(mmiSelectPrev);

        mmiSelectNext.setText(Localizer.localize("GefDemo", "MainFrame.mmiSelectNext"));
        mmEdit.add(mmiSelectNext);

        mmiInvertSelection.setText(Localizer.localize("GefDemo", "MainFrame.mmiInvertSelection"));
        mmEdit.add(mmiInvertSelection);
        mmEdit.addSeparator();

        mmiCopy.setText(Localizer.localize("GefDemo", "MainFrame.mmiCopy"));
        mmiCopy.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmiCopyMnemonic").charAt(0));
        KeyStroke ctrlC = KeyStroke.getKeyStroke(67, 2);
        mmiCopy.setAccelerator(ctrlC);
        mmEdit.add(mmiCopy);

        mmiPaste.setText(Localizer.localize("GefDemo", "MainFrame.mmiPaste"));
        mmiPaste.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmiPasteMnemonic").charAt(0));
        KeyStroke ctrlV = KeyStroke.getKeyStroke(86, 2);
        mmiPaste.setAccelerator(ctrlV);
        mmEdit.add(mmiPaste);

        KeyStroke delKey = KeyStroke.getKeyStroke(127, 0);
        mmiDelete.setAccelerator(delKey);
        mmiDelete.setText(Localizer.localize("GefDemo", "MainFrame.mmiDelete"));
        mmEdit.add(mmiDelete);

        mmiAdjustGrid.setText(Localizer.localize("GefDemo", "MainFrame.mmiAdjustGrid"));
        mmView.add(mmiAdjustGrid);
        mmiAdjustGuide.setText(Localizer.localize("GefDemo", "MainFrame.mmiAdjustGuide"));
        mmView.add(mmiAdjustGuide);
        mmiAdjustPageBreaks.setText(Localizer.localize("GefDemo", "MainFrame.mmiAdjustPageBreaks"));
        mmView.add(mmiAdjustPageBreaks);
        mmView.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmViewMnemonic").charAt(0));
        mmView.setText(Localizer.localize("GefDemo", "MainFrame.mmView"));
        fMenuBar.add(mmView);

        mmArrange.setText(Localizer.localize("GefDemo", "MainFrame.mmArrange"));
        mmArrange.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmArrangeMnemonic").charAt(0));

        mmiGroup.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmiGroupMnemonic").charAt(0));
        mmArrange.add(mmiGroup);
        KeyStroke ctrlG = KeyStroke.getKeyStroke(71, 2);
        mmiGroup.setText(Localizer.localize("GefDemo", "MainFrame.mmiGroup"));
        mmiGroup.setAccelerator(ctrlG);

        KeyStroke ctrlU = KeyStroke.getKeyStroke(85, 2);
        mmiUngroup.setAccelerator(ctrlU);
        mmiUngroup.setText(Localizer.localize("GefDemo", "MainFrame.mmiUngroup"));
        mmiUngroup.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmiUngroupMnemonic").charAt(0));
        mmArrange.add(mmiUngroup);
        fMenuBar.add(mmArrange);
        mmArrange.addSeparator();

        mmAlign.setText(Localizer.localize("GefDemo", "MainFrame.mmAlign"));
        mmiAlignTops.setText(Localizer.localize("GefDemo", "MainFrame.mmiAlignTops"));
        mmAlign.add(mmiAlignTops);
        mmiAlignBottoms.setText(Localizer.localize("GefDemo", "MainFrame.mmiAlignBottoms"));
        mmAlign.add(mmiAlignBottoms);
        mmiAlignLefts.setText(Localizer.localize("GefDemo", "MainFrame.mmiAlignLefts"));
        mmAlign.add(mmiAlignLefts);
        mmiAlignRights.setText(Localizer.localize("GefDemo", "MainFrame.mmiAlignRights"));
        mmAlign.add(mmiAlignRights);
        mmiAlignHorizCenters.setText(Localizer.localize("GefDemo", "MainFrame.mmiAlignHorizCenters"));
        mmAlign.add(mmiAlignHorizCenters);
        mmiAligVerticalCenters.setText(Localizer.localize("GefDemo", "MainFrame.mmiAligVerticalCenters"));
        mmAlign.add(mmiAligVerticalCenters);
        mmiAlignToGrid.setText(Localizer.localize("GefDemo", "MainFrame.mmiAlignToGrid"));
        mmAlign.add(mmiAlignToGrid);
        mmArrange.add(mmAlign);

        mmReorder.setText(Localizer.localize("GefDemo", "MainFrame.mmReorder"));
        mmReorder.setMnemonic(Localizer.localize("GefDemo", "MainFrame.mmReorderMnemonic").charAt(0));

        KeyStroke sCtrlF = KeyStroke.getKeyStroke(70, 3);
        mmiToFront.setText(Localizer.localize("GefDemo", "MainFrame.mmiToFront"));
        mmiToFront.setAccelerator(sCtrlF);
        mmReorder.add(mmiToFront);

        KeyStroke sCtrlB = KeyStroke.getKeyStroke(66, 3);
        mmiToBack.setText(Localizer.localize("GefDemo", "MainFrame.mmiToBack"));
        mmiToBack.setAccelerator(sCtrlB);
        mmReorder.add(mmiToBack);

        KeyStroke ctrlF = KeyStroke.getKeyStroke(70, 2);
        mmiForward.setAccelerator(ctrlF);
        mmiForward.setText(Localizer.localize("GefDemo", "MainFrame.mmiForward"));
        mmReorder.add(mmiForward);

        KeyStroke ctrlB = KeyStroke.getKeyStroke(66, 2);
        mmiBackward.setAccelerator(ctrlB);
        mmiBackward.setText(Localizer.localize("GefDemo", "MainFrame.mmiBackward"));
        mmReorder.add(mmiBackward);

        mmArrange.add(mmReorder);

        this.getContentPane().setLayout(borderLayout1);
        this.setJMenuBar(fMenuBar);

        fGraphPanel.add(fGraph, "Center");
        fGraphPanel.setBorder(new EtchedBorder(1));
        fMainPanel.add(fGraphPanel, "Center");

        fTabbedPanePalletes.addTab(Localizer.localize("GefDemo", "MainFrame.tlbNetwork"), tlbNetwork);
        fTabbedPanePalletes.addTab(Localizer.localize("GefDemo", "MainFrame.tlbCommon"), tlbCommon);
        //SamplePallete from org.tigris.gef.demo
        fTabbedPanePalletes.addTab(Localizer.localize("GefDemo", "MainFrame.tlbSamplePalette"), samplePalette);        
        fTabbedPanePalletes.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
        this.getContentPane().add(fTabbedPanePalletes, BorderLayout.NORTH);
        this.getContentPane().add(fMainPanel, BorderLayout.CENTER);
        this.getContentPane().add(fStatusBar, BorderLayout.SOUTH);
    }


    public Object clone() {
        return null;
    }


    /**
     *  Gets the graph of the MainFrame
     *
     *@return    The graph value
     */
    public JGraph getGraph() {
        return fGraph;
    }


    /**
     *  Gets the graphEdgeRenderer attribute of the MainFrame object
     *
     *@return    The graphEdgeRenderer value
     */
    public GraphEdgeRenderer getGraphEdgeRenderer() {
        return fGraph.getEditor().getGraphEdgeRenderer();
    }


    /**
     *  Gets the graphModel attribute of the MainFrame object
     *
     *@return    The graphModel value
     */
    public GraphModel getGraphModel() {
        return fGraph.getGraphModel();
    }


    /**
     *  Gets the graphNodeRenderer attribute of the MainFrame object
     *
     *@return    The graphNodeRenderer value
     */
    public GraphNodeRenderer getGraphNodeRenderer() {
        return fGraph.getEditor().getGraphNodeRenderer();
    }


    /**
     *  Gets the jMenuBar attribute of the MainFrame object
     *
     *@return    The jMenuBar value
     */
    public JMenuBar getJMenuBar() {
        return fMenuBar;
    }


    /**
     *  Sets the graph attribute of the MainFrame object
     *
     *@param  graph  The new graph value
     */
    public void setGraph(JGraph graph) {
        fGraph = graph;
    }


    /**
     *  Sets the graphEdgeRenderer attribute of the MainFrame object
     *
     *@param  rend  The new graphEdgeRenderer value
     */
    public void setGraphEdgeRenderer(GraphEdgeRenderer rend) {
        fGraph.getEditor().setGraphEdgeRenderer(rend);
    }


    /**
     *  Sets the graphModel attribute of the MainFrame object
     *
     *@param  gm  The new graphModel value
     */
    public void setGraphModel(GraphModel gm) {
        fGraph.setGraphModel(gm);
    }


    /**
     *  Sets the graphNodeRenderer attribute of the MainFrame object
     *
     *@param  rend  The new graphNodeRenderer value
     */
    public void setGraphNodeRenderer(GraphNodeRenderer rend) {
        fGraph.getEditor().setGraphNodeRenderer(rend);
    }


    /**
     *  Sets the visible attribute of the MainFrame object
     *
     *@param  visible  The new visible value
     */
    public void setVisible(boolean visible) {
        super.setVisible(visible);
        if (visible) {
            Globals.setStatusBar(this);
        }
    }


    /**
     *  Description of the Method
     *
     *@param  msg  Description of the Parameter
     */
    public void showStatus(String msg) {
        if (fStatusBar != null) {
            fStatusBar.setText(msg);
        }
    }


    private BorderLayout borderLayout1 = new BorderLayout();
    private JGraph fGraph;

    private JLabel fStatusBar = new JLabel(" ");
    private JPanel fMainPanel = new JPanel(new BorderLayout());
    private JPanel fGraphPanel = new JPanel(new BorderLayout());
    private JMenuBar fMenuBar = new JMenuBar();

    private JMenu mmFile = new JMenu();
    
    private JMenuItem mmiOpenPGML = new JMenuItem(new CmdOpenPGML());
    private JMenuItem mmiSavePGML = new JMenuItem(new CmdSavePGML("*.pgml"));
    
    private JMenuItem mmiPrint = new JMenuItem(new CmdPrint());
    private JMenuItem mmiPageSetup = new JMenuItem(new CmdPrintPageSetup(new CmdPrint()));
    private JMenuItem mmiExit = new JMenuItem(new CmdExit());

    private JMenu mmEdit = new JMenu();

    private JMenuItem mmiSelectAll = new JMenuItem(new CmdSelectAll());
    private JMenuItem mmiSelectPrev = new JMenuItem(new CmdSelectNext(false));
    private JMenuItem mmiSelectNext = new JMenuItem(new CmdSelectNext(true));
    private JMenuItem mmiInvertSelection = new JMenuItem(new CmdSelectInvert());

    private JMenu mmView = new JMenu();
    private JMenu mmArrange = new JMenu();
    private JMenu mmAlign = new JMenu();
    private JMenu mmReorder = new JMenu();

    private JMenuItem mmiCopy = new JMenuItem(new CmdCopy());
    private JMenuItem mmiPaste = new JMenuItem(new CmdPaste());
    private JMenuItem mmiDelete = new JMenuItem(new CmdDelete());

    private JMenuItem mmiAdjustGrid = new JMenuItem(new CmdAdjustGrid());
    private JMenuItem mmiAdjustGuide = new JMenuItem(new CmdAdjustGuide());
    private JMenuItem mmiAdjustPageBreaks = new JMenuItem(new CmdAdjustPageBreaks());
    private JMenuItem mmiGroup = new JMenuItem(new CmdGroup());
    private JMenuItem mmiUngroup = new JMenuItem(new CmdUngroup());

    private JMenuItem mmiAlignTops = new JMenuItem(new CmdAlign(0));
    private JMenuItem mmiAlignBottoms = new JMenuItem(new CmdAlign(1));
    private JMenuItem mmiAlignLefts = new JMenuItem(new CmdAlign(2));
    private JMenuItem mmiAlignRights = new JMenuItem(new CmdAlign(3));
    private JMenuItem mmiAlignHorizCenters = new JMenuItem(new CmdAlign(5));
    private JMenuItem mmiAligVerticalCenters = new JMenuItem(new CmdAlign(6));
    private JMenuItem mmiAlignToGrid = new JMenuItem(new CmdAlign(7));

    private JMenuItem mmiToBack = new JMenuItem(new CmdReorder(1));
    private JMenuItem mmiToFront = new JMenuItem(new CmdReorder(2));
    private JMenuItem mmiBackward = new JMenuItem(new CmdReorder(3));
    private JMenuItem mmiForward = new JMenuItem(new CmdReorder(4));

    private static PaletteFig tlbCommon = new PaletteFig();
    private static NetworkToolbar tlbNetwork = new NetworkToolbar();
    private static SamplePalette samplePalette = new SamplePalette();

    private static JTabbedPane fTabbedPanePalletes = new JTabbedPane();
}
